# Begin TickleServices Version 1.02mab Data
"Divider" = "155.000000";
"Menu Item" = "!Initialization!";
"Tcl" = "\
# This !Initialization! service was taken pretty much straight from t\
he
#  TickleServices.ts !Initialization! with a modification of 'nexec' \
to make
# it return any error messages it 'catch'es

    # This version of exec doesn't strip the trailing newline from th\
e
    # output, and returns the 'catch'ed stuff as well as the result
    # so we can have some smart error handling in the services
proc nexec {args} {
    global errorInfo

    set fileId [uplevel open [list \"|$args\"]]
    set ret [read $fileId]
    set foo [catch {close $fileId} caught]

    return [list $ret $caught]
}

proc filenames {} {
    global NXFilenamePboardType
    return [split [pasteboard read $NXFilenamePboardType] \"\\t\"]
}

    # Reads the specified file and returns the text.
proc readfile {filename {hexcode 0}} {
    set fileId [open $filename r]
    if {$hexcode} {
        set ret [read -hexcode $fileId]
    } else {
        set ret [read $fileId]
    }
    close $fileId
    return $ret
}

    # Writes the text to the specified file.
proc writefile {filename text {hexcode 0}} {
    set fileId [open $filename w]
    if {$hexcode} {
        set ret [puts -nonewline -hexcode $fileId $text]
    } else {
        set ret [puts -nonewline $fileId $text]
    }
    close $fileId
    return $ret
}

    # read the quoteString from the defaults database.  The quoteStri\
ng
    # defaults to \">\", which is a semi-standard quoting character.
global quoteString
set quoteString [defaults read TickleServices QuoteString \">\"]

    # read the indentString from the defaults database.  The indentSt\
ring
    # defaults to \"    \".
global indentString
set indentString [defaults read TickleServices IndentString \"    \"]

    # Quote the passed string using quoteString and return it.
    # Don't modify the regsub unless you _really_ know what you're
    # doing.  It's sort of evil ... I match the stuff in the line and
    # the end-of-line (optional because the last line may not have
    # one), and then replace the front of the line with the quoteStri\
ng.
    # Escape \\ and & in the quote string just in case.
proc quote {str {qs {}}} {
    if {![string length $qs]} {
        global quoteString
        set qs $quoteString
    }
    if {[regsub -all \"^(\\[^\\n\\]*)(\\n?)\" $str \"[escape $qs {\\\\\
&}]\\\\1\\\\2\" foo]} {
	return $foo
    } else {
	return $str
    }
}

    # Strip quotes from the passed string.
    # Don't modify the regsub unless you _really_ know what you're
    # doing.  It's sort of evil ... I match the quote string at the
    # front, then the stuff in the line, and then the end-of-line
    # (optional because the last line may not have one), and replace
    # that with just the in-the-line stuff plus the end-of-line.
proc unquote {str {qs {}}} {
    if {![string length $qs]} {
        global quoteString
        set qs $quoteString
    }
    if {[regsub -all \"^[escape $qs {\\\\()[]^*$?}](\\[^\\n\\]*)(\\n?\
)\" $str \"\\\\1\\\\2\" foo]} {
	return $foo
    } else {
	return $str
    }
}

    # Expand tabs to spaces.
proc expand {str} {
    return [nexec expand << $str]
}

    # Unexpand spaces to tabs.
proc unexpand {str} {
    return [nexec unexpand << $str]
}

    # Reformat the passed string.
proc fmt {str {width 66}} {
    return [nexec /usr/ucb/fmt $width [expr {$width+8}] << $str]
}
";
"Window Size" = "{ 847.000, 480.000}";
|
"Menu Item" = "PGP/Add Key to Keyring";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# PGP/Add Key
# Copyright  (c) 1993 Brian Gottlieb
# Brian Gottlieb (brian@siesta.wustl.edu)
# 29SEP93
#
# Uses PGP ((c) 1990-1993 Philip Zimmermann, Phil's Pretty Good Softw\
are.)
# to add a selected keyfile to your public keyring

# Call PGP as a filter, in batchmode (so it doesn't ask any questions\
)
set cmd \"pgp -kaf  +interactive=off +batchmode\"
set result [eval [concat nexec $cmd << [list [append  a \"\\n\" [past\
eboard read]]]]]
unset a

alertpanel {PGP Add Keyfile}  [lindex $result 1] OK \"Well, this just\
 honks my hooter!\"

unset result


";
"Timeout" = "60000";
|
"Menu Item" = "PGP/Decrypt and View...";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# PGP/Decrypt...
# Copyright  (c) 1993 Brian Gottlieb
# Brian Gottlieb (brian@siesta.wustl.edu)
# 17SEP93
#
# Uses PGP ((c) 1990-1993 Philip Zimmermann, Phil's Pretty Good Softw\
are.)
# to Decrypt a PGP Crypted message
# Notifies user of any signatures attached to the message

# prompt for pass phrase
set passphrase [promptpanel -password -message {Enter pass phrase} {P\
GP Decrypt Selection}]
if {$passphrase==\"\"} {
	return
}
set passphrase \"$passphrase\\n\"

# Set up the environment so PGP expects the password as the first lin\
e of the input
set env(PGPPASSFD) 0

# Call PGP as a filter, in batchmode (so it doesn't ask any questions\
)
# Pass it the password, followed by the message
set cmd \"pgp -f +batchmode\"
set result [eval [concat nexec $cmd << [list [append  passphrase [pas\
teboard read]]]]]

# wipe out the passphrase, hopefully
set passphrase \"ABCDEFGHIJKLMNOPQRSTUVWXYZ\"

# check to see if it decrypted OK
set plaintext [lindex $result 0]
if {$plaintext != \"\"} {
	# Check for signatures
	if {[regexp -nocase good\\ signature.* [lindex $result 1]  matched] \
} {
		alertpanel {Valid PGP Signature found...} $matched OK Groovy
	} elseif {[regexp -nocase warning.* [lindex $result 1]  matched] } {
		alertpanel {Bad PGP Signature found...} $matched OK Hmm...
	}
	alertpanel {PGP Decrypted Message}  $plaintext OK {I am done reading\
 this message}
	#listpanel -empty -message {} {PGPDecrypted Message} [fmt $plaintext\
 40 ] OK 
} else { # error occurred
    	alertpanel {PGP Decryption Error}  [lindex $result 1] OK {That s\
ucks!}
}

";
"Timeout" = "60000";
|
"Menu Item" = "PGP/Decrypt...";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# PGP/Decrypt...
# Copyright  (c) 1993 Brian Gottlieb
# Brian Gottlieb (brian@siesta.wustl.edu)
# 17SEP93
#
# Uses PGP ((c) 1990-1993 Philip Zimmermann, Phil's Pretty Good Softw\
are.)
# to Decrypt a PGP Crypted message
# Notifies user of any signatures attached to the message

# prompt for pass phrase
set passphrase [promptpanel -password -message {Enter pass phrase} {P\
GP Decrypt Selection}]
if {$passphrase==\"\"} {
	return
}
set passphrase \"$passphrase\\n\"

# Set up the environment so PGP expects the password as the first lin\
e of the input
set env(PGPPASSFD) 0

# Call PGP as a filter, in batchmode (so it doesn't ask any questions\
)
# Pass it the password, followed by the message
set cmd \"pgp -f +batchmode\"
set result [eval [concat nexec $cmd << [list [append  passphrase [pas\
teboard read]]]]]

# wipe out the passphrase, hopefully
set passphrase \"ABCDEFGHIJKLMNOPQRSTUVWXYZ\"

# check to see if it decrypted OK
set plaintext [lindex $result 0]
if {$plaintext != \"\"} {
	# Check for signatures
	if {[regexp -nocase good\\ signature.* [lindex $result 1]  matched] \
} {
		alertpanel {Valid PGP Signature found...} $matched OK Groovy
	} elseif {[regexp -nocase warning.* [lindex $result 1]  matched] } {
		alertpanel {Bad PGP Signature found...} $matched OK Hmm...
	}
	pasteboard write $plaintext
} else { # error occurred
    	alertpanel {PGP Decryption Error}  [lindex $result 1] OK {That s\
ucks!}
}

";
"Timeout" = "60000";
|
"Menu Item" = "PGP/Digital Signature...";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# PGP/Digital Signature...
# Copyright  (c) 1993 Brian Gottlieb
# Brian Gottlieb (brian@siesta.wustl.edu)
# 17SEP93
#
# Uses PGP ((c) 1990-1993 Philip Zimmermann, Phil's Pretty Good Softw\
are.)
# to sign a plaintext message with your PGP digital signature

# prompt for pass phrase
set passphrase [promptpanel -password -message {Enter pass phrase} {P\
GP Sign Selection}]
if {$passphrase==\"\"} {
	return
}
set passphrase \"$passphrase\\n\"

# Set up the environment so PGP expects the password as the first lin\
e of the input
set env(PGPPASSFD) 0

# Call PGP as a filter, in batchmode (so it doesn't ask any questions\
)
# Pass it the password, followed by the message
set cmd \"pgp -fast +clearsig=on +batchmode\"
set result [eval [concat nexec $cmd << [list [append  passphrase [lin\
dex [fmt [pasteboard read]] 0]]]]]

# wipe out the passphrase, hopefully
set passphrase \"ABCDEFGHIJKLMNOPQRSTUVWXYZ\"

# Make sure everything worked
set plaintext [lindex $result 0]
if {$plaintext != \"\"} {
	pasteboard write $plaintext
} else { # error
   	alertpanel {PGP Digital Signature Error}  [lindex $result 1]
}


";
"Timeout" = "60000";
|
"Menu Item" = "PGP/Encrypt and Sign...";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# PGP/Encrypt and Sign...
# Copyright  (c) 1993 Brian Gottlieb
# Brian Gottlieb (brian@siesta.wustl.edu)
# 17SEP93
#
# Uses PGP ((c) 1990-1993 Philip Zimmermann, Phil's Pretty Good Softw\
are.)
# to encrypt and sign a message with your PGP digital signature

# prompt for people to encrypt for
set people [promptpanel -message {Enter people to encrypt for} {PGP E\
ncrypt and Sign Selection}]
if {$people==\"\"} {
	return
}

# prompt for pass phrase
set passphrase [promptpanel -password -message {Enter pass phrase} {P\
GP Encrypt and Sign Selection}]
if {$passphrase==\"\"} {
	return
}
set passphrase \"$passphrase\\n\"

# Set up the environment so PGP expects the password as the first lin\
e of the input
set env(PGPPASSFD) 0

# Call PGP as a filter, in batchmode (so it doesn't ask any questions\
)
# Pass it the password, followed by the message
set cmd \"pgp -feast +batchmode $people\"
set result [eval [concat nexec $cmd << [list [append  passphrase [lin\
dex [fmt [pasteboard read]] 0]]]]]

# wipe out the passphrase, hopefully
set passphrase \"ABCDEFGHIJKLMNOPQRSTUVWXYZ\"

# Make sure it all worked
set plaintext [lindex $result 0]
if {$plaintext != \"\"} {
	pasteboard write $plaintext
} else { # error
    	alertpanel {PGP Encryption/Signing Error}  [lindex $result 1]
}
";
"Timeout" = "60000";
|
"Menu Item" = "PGP/Encrypt...";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# PGP/Encrypt...
# Copyright  (c) 1993 Brian Gottlieb
# Brian Gottlieb (brian@siesta.wustl.edu)
# 17SEP93
#
# Uses PGP ((c) 1990-1993 Philip Zimmermann, Phil's Pretty Good Softw\
are.)
# to encrypt  a message with PGP

# prompt for people to encrypt for
set people [promptpanel -message {Enter people to encrypt for} {PGP E\
ncrypt Selection}]
if {$people==\"\"} {
	return
}


# Call PGP as a filter, in batchmode (so it doesn't ask any questions\
)
set cmd \"pgp -feat +batchmode $people\"
set result [eval [concat nexec $cmd << [fmt [append  a \"\\n\" [paste\
board read]]]]]

# just so $a doesn't screw up next time
unset a

# Make sure it worked right
set plaintext [lindex $result 0]
if {$plaintext != \"\"} {
	pasteboard write $plaintext
} else { # error
    	alertpanel {PGP EncryptionError}  [lindex $result 1]
}
";
"Timeout" = "60000";
|
"Menu Item" = "PGP/Extract Key...";
"Receive Type" = "NXAsciiPboardType";
"Tcl" = "\
# PGP/Extract Key
# Copyright  (c) 1993 Brian Gottlieb
# Brian Gottlieb (brian@siesta.wustl.edu)
# 29SEP93
#
# Uses PGP ((c) 1990-1993 Philip Zimmermann, Phil's Pretty Good Softw\
are.)
# to insert someone's public key in a message

# prompt for user
set user [promptpanel -message {Enter user to extract} {PGP Extract K\
ey}]
if {$user==\"\"} {
	return
}
# Call PGP as a filter, in batchmode (so it doesn't ask any questions\
)
set cmd \"pgp -fkxa +batchmode $user\"
set result [eval [concat nexec $cmd ]]

set plaintext [lindex $result 0]

if {$plaintext != \"\"} {
	pasteboard write $plaintext
} else { # error
    	alertpanel {PGP Public Key Extraction Error}  [lindex $result 1]
}

unset result";
"Timeout" = "60000";
|
"Menu Item" = "PGP/Insert Key";
"Receive Type" = "NXAsciiPboardType";
"Tcl" = "\
# PGP/Insert Key
# Copyright  (c) 1993 Brian Gottlieb
# Brian Gottlieb (brian@siesta.wustl.edu)
# 29SEP93
#
# Uses PGP ((c) 1990-1993 Philip Zimmermann, Phil's Pretty Good Softw\
are.)
# to insert your public key in a message

# Call PGP as a filter, in batchmode (so it doesn't ask any questions\
)
set cmd \"pgp -fkxa +batchmode $env(USER)\"
set result [eval [concat nexec $cmd ]]

set plaintext [lindex $result 0]

if {$plaintext != \"\"} {
	pasteboard write $plaintext
} else { # error
    	alertpanel {PGP Public Key Extraction Error}  [lindex $result 1]
}

unset result";
"Timeout" = "60000";
|
"Menu Item" = "PGP/Insert Keyring";
"Receive Type" = "NXAsciiPboardType";
"Tcl" = "\
# PGP/Insert Keyring
# Copyright  (c) 1993 Brian Gottlieb
# Brian Gottlieb (brian@siesta.wustl.edu)
# 29SEP93
#
# Uses PGP ((c) 1990-1993 Philip Zimmermann, Phil's Pretty Good Softw\
are.)
# to insert keys from your keyring into a message

# Call PGP as a filter, in batchmode (so it doesn't ask any questions\
)
set cmd \"pgp -fkxa +batchmode\"
set result [eval [concat nexec $cmd ]]

set plaintext [lindex $result 0]

if {$plaintext != \"\"} {
	pasteboard write $plaintext
} else { # error
    	alertpanel {PGP Public Key Extraction Error}  [lindex $result 1]
}

unset result";
"Timeout" = "60000";
|
"Menu Item" = "PGP/Strip Signature";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# PGP/Strip Signature
# Copyright  (c) 1993 Brian Gottlieb
# Brian Gottlieb (brian@siesta.wustl.edu)
# 17SEP93
#
# Uses PGP ((c) 1990-1993 Philip Zimmermann, Phil's Pretty Good Softw\
are.)
# to remove a signature from a signed plaintext message

# Call PGP as a filter, in batchmode (so it doesn't ask any questions\
)
set cmd \"pgp -f +batchmode\"
set result [eval [concat nexec $cmd << [list [append  a \"\\n\" [past\
eboard read]]]]]
unset a

# Make sure it all worked
set plaintext [lindex $result 0]
if {$plaintext != \"\"} {
	# check the signature
	if {[regexp -nocase good\\ signature.* [lindex $result 1]  matched] \
} {
		alertpanel {Valid PGP Signature found...} $matched OK Groovy
		pasteboard write $plaintext
	} elseif {[regexp -nocase warning.* [lindex $result 1]  matched] } {
		alertpanel {Bad PGP Signature found...} $matched OK Hmm...
	} else { # unknown signature error
		alertpanel {PGP Signature Verification Error}  [lindex $result 1] O\
K Suck!
	}
} else { # error
	alertpanel {PGP Signature Removal Error}  [lindex $result 1] OK Suck\
!
}


#pasteboard write [list [append  passphrase [pasteboard read]]]

";
"Timeout" = "60000";
|
"Menu Item" = "PGP/Verify Signature";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# PGP/Verify Signature
# Copyright  (c) 1993 Brian Gottlieb
# Brian Gottlieb (brian@siesta.wustl.edu)
# 17SEP93
#
# Uses PGP ((c) 1990-1993 Philip Zimmermann, Phil's Pretty Good Softw\
are.)
# to verify a signature from a signed plaintext message

# Call PGP as a filter, in batchmode (so it doesn't ask any questions\
)
set cmd \"pgp -f +batchmode\"
set result [eval [concat nexec $cmd << [list [append  a \"\\n\" [past\
eboard read]]]]]
unset a

# Check the validity
if {[regexp -nocase good\\ signature.* [lindex $result 1]  matched] }\
 { # good signature
		alertpanel {Valid PGP Signature found...} $matched OK Groovy
} elseif {[regexp -nocase warning.* [lindex $result 1]  matched] } { \
# bad signature
		alertpanel {Bad PGP Signature found...} $matched OK Hmm...
} else { # error
		alertpanel {PGP Signature Verification}  [lindex $result 1] OK Suck\
!
}

unset result


";
"Timeout" = "60000";
# End TickleServices Data
