/* -*-C-*-
*******************************************************************************
*
* File:         appnmail.m
* RCS:          $Header: /usr/local/lib/cvs/mailapp-utilities/appnmail.m,v 1.12 1996/03/31 16:13:55 cedman Exp $
* Description:  Append stdin to Mail.app mailbox
* Author:       Carl Edman
* Created:      Fri Mar 12 18:21:23 1993
* Modified:     Sat Mar 30 19:10:17 1996 (Carl Edman) cedman@capitalist.princeton.edu
* Language:     Objective C
* Package:      mailapp-utilities
* Status:       First release.
*
* (C) Copyright 1993, but otherwise this file is perfect freeware.
*
*******************************************************************************
*/

#import <appkit/appkit.h>
#import <stdlib.h>
#import <stdio.h>
#import <string.h>
#import <regex.h>
#import <sys/file.h>
#import <sys/param.h>
#import <sys/types.h>
#import <sys/stat.h>
#import <defaults/defaults.h>
#import "mailutil.h"
#import "mailapp.h"

char *header=0,*content=0;
int headerlen=0,contentlen=0;
int headermaxlen=0,contentmaxlen=0;
char line[LINELEN];

int readmail(void)
   {
   int i,pri=0;
   char from[LINELEN]="",subject[LINELEN]="",reference[LINELEN]="";
   struct regex *fre=re_compile("^From:  *\\(.*\\)\n",0);
   struct regex *sre=re_compile("^Subject:  *\\(.*\\)\n",0);
   struct regex *dre=re_compile("^Date: .*\\([ 0123][0-9]\\) \\([A-Z][a-z][a-z][a-z]*\\) 1*9*\\([789][0-9]\\)",0);
   struct regex *rre=re_compile("^Next-Attachment: \\.tar\\.\\([0-9]*\\)\\.\\(.*\\)\\.attach, \\(E*,* *\\)\\([0-9]*\\), \\([0-9]*/[0-9]*\\), \\([0-9]*\\), \\([0-9]*\\)",0);
   struct regex *mre=re_compile("^Content-Type: \\([a-zA-Z]*/[a-zA-Z]*\\)",0);
   struct regex *pre=0;
   const char *pridef=0;

   if (pridef=NXGetDefaultValue("Mail","PriorityHeader"))
      {
      static char buf[MAXPATHLEN];
      sprintf(buf,"^%s: [ \t]*\\([^ \t]*\\)[ \t]*\n",pridef);
      pre=re_compile(buf,0);
      pridef=NXGetDefaultValue("Mail","PriorityValues");
      }
   
   mi=malloc(sizeof(*mi));
   
   mi->record_length=sizeof(*mi);
   mi->mes_offset=-1;
   mi->status='*';
   mi->msgtype=' ';
   mi->encrypted=' ';
   mi->sync=' ';
   mi->mes_date=message_current_date();

   headerlen=0;
   
   while(fgets(line,LINELEN,stdin))
      {
      if (*line=='\n') break;
      if (re_match(line,fre)==1)
         strpcpy(from,fre->braslist[0],fre->braelist[0]);
      if (re_match(line,sre)==1)
         strpcpy(subject,sre->braslist[0],sre->braelist[0]);
      if (re_match(line,dre)==1)
         mi->mes_date=message_date(atoi(dre->braslist[2])+1900,
                                   dre->braslist[1],atoi(dre->braslist[0]));
      if (pre && pridef && (re_match(line,pre)==1))
         {
         const char *beg,*end;

         for(beg=pridef,pri=1;*beg;(beg=*end ? end+1 : end),(pri++))
            {
            if (!(end=index(beg,' '))) end=beg+strlen(beg);
            if (((pre->braelist[0]-pre->braslist[0])==(end-beg))
                &&(strncmp(pre->braslist[0],beg,end-beg)==0))
               break;
            }
         if (!*beg) pri=0;
         }
      if ((mi->msgtype==' ') && (re_match(line,mre)==1))
         {
         if (!strpcaseequ("text/plain",mre->braslist[0],mre->braelist[0]))
            mi->msgtype='m';
         }
      if (re_match(line,rre)==1)
         {
         char *c;
         mi->msgtype='r';
         strpcpy(reference,rre->braslist[1],rre->braelist[1]);
         c=reference+strlen(reference);
         *c++='_';
         while(c-reference<22) *c++='_';
         *c='\0';
         sprintf(c,"%ld",time(0));
         strcat(reference,".attach");
         c=reference+strlen(reference);
         strcat(reference,", ");
         if (*(rre->braslist[2])=='E')
            {
            strcat(reference,"E, ");
            mi->encrypted='e';
            }
         strpcat(reference,rre->braslist[4],rre->braelist[4]);
         strcat(reference,"\n");
         appstring(&header,&headerlen,&headermaxlen,"Next-Reference: ",
                   strlen("Next-Reference: "));
         appstring(&header,&headerlen,&headermaxlen,reference,strlen(reference));
         *c='\0';
         }
      else
         {
         appstring(&header,&headerlen,&headermaxlen,line,strlen(line));
         }
      }
   appstring(&header,&headerlen,&headermaxlen,"\n",1);

   contentlen=0;
   
   while((i=fread(growstring(&content,&contentlen,&contentmaxlen,LINELEN*16),
                  1,16*LINELEN,stdin))>0)
      contentlen+=i;
   if (content[contentlen-1]!='\n')
      appstring(&content,&contentlen,&contentmaxlen,"\n",1);

   mi->mes_length=headerlen;
   if (mi->msgtype!='r') mi->mes_length+=contentlen;
   mi->record_length+=strlen(from)+1+strlen(subject)+1+strlen(reference)+1;
   mi->record_length+=sizeof(int)+sizeof(time_t)+sizeof(int);
   mi=realloc(mi,mi->record_length);
   strcpy(mi->data,from);
   strcpy(mi->data+strlen(from)+1,subject); 
   strcpy(mi->data+strlen(from)+1+strlen(subject)+1,reference);

   message_set_attachsize(mi,0);
   message_set_attachtime(mi,0);
   message_set_priority(mi,pri);
   
   free(fre); free(sre); free(rre); free(dre);
   if (pre) free(pre);
   return 0;
   }

void main(int ac,char *av[])
   {
   int c;
   int errflg=0,readflg=0,nowaitflg=0,verboseflg=0,multiflg=0,flagflg=0,deleteflg=0;
   int pri=-1;
   time_t initmtime;
   int mfd,tfd;
   struct table_of_contents_header *toch=0;
   while((c=getopt(ac,av,"rfdvnmp:"))!=EOF) switch(c)
      {
    case 'r':
      readflg++;
      if (flagflg||deleteflg) errflg++;
      break;
    case 'f':
      flagflg++;
      if (readflg||deleteflg) errflg++;
      break;
    case 'd':
      deleteflg++;
      if (readflg||flagflg) errflg++;
      break;
    case 'n':
      nowaitflg++;
      break;
    case 'v':
      verboseflg++;
      break;
    case 'm':
      multiflg++;
      break;
    case 'p':
      if (pri!=-1) errflg++;
      pri=atoi(optarg);
      if (pri<0) errflg++;
      break;
    case '?':
    default:
      errflg++;
      }

   if (errflg)
      {
      fprintf(stderr,"Usage: %s [-nv][-r|-f|-d|][-p <pri>] [mbox...]\n",av[0]);
      exit(1);
      }

   readmail();

   if (!header || (headerlen<5) || strncmp(header,"From ",5))
      {
      fprintf(stderr,"Message is not in UNIX mailbox format\n");
      exit(2);
      }

   if (readflg) mi->status=' ';
   else if (flagflg) mi->status='+';
   else if (deleteflg) mi->status='d';

   if (pri>=0) message_set_priority(mi,pri);
   
   for(;optind<ac;optind++)
      {
      if (verboseflg) fprintf(stderr,"Entering %s...\n",av[optind]);
      if (cd_mbox(av[optind],1)) goto next;
      if (lock_mbox(nowaitflg)) goto next;

      initmtime=mtime("mbox");
      
      if ((mfd=open("mbox",O_WRONLY|O_APPEND|O_CREAT,0644))==-1)
         {
         fprintf(stderr,"%s: %s\n",av[optind],strerror(errno));
         goto unlock;
         }

      if ((tfd=open("table_of_contents",O_CREAT|O_RDWR,0644))==-1)
         {
         fprintf(stderr,"%s: %s\n",av[optind],strerror(errno));
         goto unlock;
         }
      toch=get_table_of_contents_header(tfd,1);
      
      /* FOR EACH */
      lseek(mfd,0,L_XTND);
      mi->mes_offset=lseek(mfd,0,L_INCR);
      
      write(mfd,header,headerlen);
      if (mi->msgtype=='r')
         {
         char path[MAXPATHLEN];
         FILE *f;

         write(mfd,"\n",1);
         strcpy(path,message_reference(mi));
         if ((mkdir(path,0755))==-1)
            {
            fprintf(stderr,"%s: %s\n",path,strerror(errno));
            goto unlock;
            }
         sprintf(line,"cd %s && /NextApps/Mail.app/decode | /usr/ucb/uncompress | /NextApps/Mail.app/safetar xf -",path);
         if (!(f=popen(line,"w")))
            {
            fprintf(stderr,"%s: %s\n",path,strerror(errno));
            goto unlock;
            }
         fwrite(content,contentlen,1,f);
         pclose(f);
         message_set_attachsize(mi,dirsize(path));
         message_set_attachtime(mi,mtime(path));
         }
      else
         {
         write(mfd,content,contentlen);
         }
      toch->num_msgs++;
      lseek(tfd,0,L_XTND);
      put_message_index(tfd,mi);
      /* END FOR EACH */
      
      close(mfd);
      if (toch->mbox_time==initmtime) toch->mbox_time=mtime("mbox");
      lseek(tfd,0,L_SET);
      put_table_of_contents_header(tfd,toch);
      close(tfd);
    unlock:
      unlock_mbox();
    next:;
      if (toch) { free(toch); toch=0; }
      uncd_mbox();
      }
   exit(0);
   }
