/* -*-C-*-
*******************************************************************************
*
* File:         compactmail.m
* RCS:          $Header: /usr/local/lib/cvs/mailapp-utilities/compactmail.m,v 1.1.1.1 1993/10/30 00:53:26 cedman Exp $
* Description:  Compact Mail.app mailboxes (expunge deleted messages)
* Author:       Carl Edman
* Created:      Fri Mar 12 18:21:23 1993
* Modified:     Sat Sep  4 15:53:45 1993 (Carl Edman) cedman@capitalist.princeton.edu
* Language:     Objective C
* Package:      mailapp-utilities
* Status:       First release.
*
* (C) Copyright 1993, but otherwise this file is perfect freeware.
*
*******************************************************************************
*/

#import <libc.h>
#import <errno.h>
#import <stdlib.h>
#import <stdio.h>
#import <string.h>
#import <time.h>
#import <regex.h>
#import <sys/file.h>
#import <sys/param.h>
#import <sys/types.h>
#import <sys/stat.h>
#import <defaults/defaults.h>
#import "mailutil.h"
#import "mailapp.h"

char line[LINELEN];

int main(int ac,char *av[])
   {
   char *buf=0;
   int c,pos;
   int errflg=0,nowaitflg=0,verboseflg=0,deletereadflg=0,deleteunreadflg=0;
   int days=-1;
   struct regex *subjectregexp=0,*fromregexp=0;
   time_t mboxtime;
   int mboxdis,tocdis,msgdis,changed;
   int mboxfd,tocfd;
   struct table_of_contents_header *toch=0;
   struct message_index *mi=0;
   
   while((c=getopt(ac,av,"d:s:f:nruv"))!=EOF) switch(c)
      {
    case 'd':
      if (days!=-1 || (days=atoi(optarg))<=0)
         errflg++;
      break;
    case 's':
      if (subjectregexp || !(subjectregexp=re_compile(optarg,0)))
         errflg++;
      break;
    case 'f':
      if (fromregexp || !(fromregexp=re_compile(optarg,0)))
         errflg++;
      break;
    case 'n':
      nowaitflg++;
      break;
    case 'r':
      deletereadflg++;
      break;
    case 'u':
      deleteunreadflg++;
      break;
    case 'v':
      verboseflg++;
      break;
    case '?':
    default:
      errflg++;
      }

   if (errflg)
      {
      fprintf(stderr,"Usage: %s [-nruv] [-d days] [-f from] [-s subject] [mbox...]\n",av[0]);
      exit(1);
      }

   for(;optind<ac;optind++)
      {
      if (verboseflg) fprintf(stderr,"Entering %s...\n",av[optind]);
      if (cd_mbox(av[optind],1)) goto next;
      if (lock_mbox(nowaitflg)) goto next;

      mboxtime=mtime("mbox");

      if (((mboxfd=open("mbox",O_RDWR))==-1) ||
          ((tocfd=open("table_of_contents",O_RDWR))==-1))
         {
         fprintf(stderr,"Opening: %s\n",strerror(errno));
         goto unlock;
         }

      toch=get_table_of_contents_header(tocfd,0);
      if (!toch)
         {
         fprintf(stderr,"No old table_of_contents\n");
         goto unlock;
         }

      if (toch->mbox_time!=mboxtime)
         {
         fprintf(stderr,"Table of contents out of sync\n");
         goto unlock;
         }

      for(changed=FALSE,msgdis=mboxdis=tocdis=c=0;c<toch->num_msgs;c++)
         {
         if ((mi=get_message_index(tocfd))==0)
            {
            fprintf(stderr,"Reading old mbox entries: %s\n",strerror(errno));
            goto unlock;
            }
         if ((mi->status=='d' || deleteunreadflg ||
              (mi->status!='*' && deletereadflg)) &&
             (days==-1 || message_age(mi)>=days) &&
             (!subjectregexp || re_match(message_subject(mi),subjectregexp)) &&
             (!fromregexp || re_match(message_from(mi),fromregexp)))
            {
            if (verboseflg>1)
               {
               fprintf(stderr,"Expunging \"%s\" from %s (%d days old).\n",message_subject(mi),message_from(mi),message_age(mi));
               }
            if (mi->msgtype=='r')
               {
               char *dir=mi->data;
               dir+=strlen(dir)+1;
               dir+=strlen(dir)+1;
               if (*dir)
                  {
                  sprintf(line,"/bin/rm -rf %s",dir);
                  system(line);
                  }               
               }
            changed=TRUE;
            msgdis++;
            tocdis +=mi->record_length;
            mboxdis+=mi->mes_length;
            }
         else
            {
            if (verboseflg>2)
               {
               fprintf(stderr,"Keeping \"%s\" from %s (%d days old).\n",message_subject(mi),message_from(mi),message_age(mi));
               }
            if (mboxdis!=0)
               {
               lseek(mboxfd,mi->mes_offset,L_SET);
               buf=malloc(mi->mes_length);
               read(mboxfd,buf,mi->mes_length);
               mi->mes_offset -= mboxdis;
               lseek(mboxfd,mi->mes_offset,L_SET);
               write(mboxfd,buf,mi->mes_length);
               free(buf); buf=0;
               }

            if (tocdis!=0 || mboxdis!=0)
               {
               pos=lseek(tocfd,0,L_INCR);
               lseek(tocfd,-mi->record_length-tocdis,L_INCR);
               put_message_index(tocfd,mi);
               lseek(tocfd,pos,L_SET);
               }
            }
         free(mi); mi=0;
         }
      
      if (tocdis!=0)
         {
         pos=lseek(tocfd,0,L_XTND);
         ftruncate(tocfd,pos-tocdis);
         }

      if (mboxdis!=0)
         {
         pos=lseek(mboxfd,0,L_XTND);
         ftruncate(mboxfd,pos-mboxdis);
         }
      close(mboxfd);
      
      toch->num_msgs-=msgdis;
      mboxtime=mtime("mbox");
      if ((toch->mbox_time!=mboxtime) || changed)
         {
         toch->mbox_time=mboxtime;
         lseek(tocfd,0,L_SET);
         if (put_table_of_contents_header(tocfd,toch))
            {
            fprintf(stderr,"Writing updated table_of_contents: %s\n",strerror(errno));
            goto unlock;
            }
         }
      close(tocfd);
    unlock:
      unlock_mbox();
    next:;
      if (toch) { free(toch); toch=0; }
      if (mi) { free(mi); mi=0; }
      if (buf) { free(buf); buf=0; }
      uncd_mbox();
      }
   exit(0);
   }
